//
//  main.m
//  CoreDataTutorial
//
//  Created by kesalin on 8/29/11.
//  Copyright 2011 kesalin@gmail.com. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <CoreData/CoreData.h>
#include <objc/objc-auto.h>
#import "Run.h"

NSURL *applicationLogDirectory()
{
    NSString *LOG_DIRECTORY = @"CoreDataTutorial";
    static NSURL *ald = nil;
    
    if (ald == nil)
    {
        NSFileManager *fileManager = [[NSFileManager alloc] init];
        NSError *error = nil;
        NSURL *libraryURL = [fileManager URLForDirectory:NSLibraryDirectory inDomain:NSUserDomainMask
                                       appropriateForURL:nil create:YES error:&error];
        if (libraryURL == nil) {
            NSLog(@"Could not access Library directory\n%@", [error localizedDescription]);
        }
        else
        {
            ald = [libraryURL URLByAppendingPathComponent:@"Logs"];
            ald = [ald URLByAppendingPathComponent:LOG_DIRECTORY];
            
            NSLog(@" >> log path %@", [ald path]);
            
            NSDictionary *properties = [ald resourceValuesForKeys:[NSArray arrayWithObject:NSURLIsDirectoryKey] error:&error];
            if (properties == nil)
            {
                if (![fileManager createDirectoryAtPath:[ald path] withIntermediateDirectories:YES attributes:nil error:&error])
                {
                    NSLog(@"Could not create directory %@\n%@",
                          [ald path], [error localizedDescription]);
                    ald = nil;
                }
            }
        }
    }
    
    return ald;
}

NSManagedObjectModel *managedObjectModel()
{
    static NSManagedObjectModel *moModel = nil;

    if (moModel != nil) {
        return moModel;
    }
    
    moModel = [[NSManagedObjectModel alloc] init];
    
    // Create the entity
    //
    NSEntityDescription *runEntity = [[NSEntityDescription alloc] init];
    [runEntity setName:@"Run"];
    [runEntity setManagedObjectClassName:@"Run"];
    
    [moModel setEntities:[NSArray arrayWithObject:runEntity]];
    
    // Add the Attributes
    //
    NSAttributeDescription *dateAttribute = [[NSAttributeDescription alloc] init];
    [dateAttribute setName:@"date"];
    [dateAttribute setAttributeType:NSDateAttributeType];
    [dateAttribute setOptional:NO];
    
    NSAttributeDescription *idAttribute = [[NSAttributeDescription alloc] init];
    [idAttribute setName:@"processID"];
    [idAttribute setAttributeType:NSInteger32AttributeType];
    [idAttribute setOptional:NO];
    [idAttribute setDefaultValue:[NSNumber numberWithInteger:-1]];

    // Create the validation predicate for the process ID.
    // The following code is equivalent to validationPredicate = [NSPredicate predicateWithFormat:@"SELF > 0"]
    //
    NSExpression *lhs = [NSExpression expressionForEvaluatedObject];
    NSExpression *rhs = [NSExpression expressionForConstantValue:[NSNumber numberWithInteger:0]];
    
    NSPredicate *validationPredicate = [NSComparisonPredicate
                                        predicateWithLeftExpression:lhs
                                        rightExpression:rhs
                                        modifier:NSDirectPredicateModifier
                                        type:NSGreaterThanPredicateOperatorType
                                        options:0];
    
    NSString *validationWarning = @"Process ID < 1";
    [idAttribute setValidationPredicates:[NSArray arrayWithObject:validationPredicate]
                  withValidationWarnings:[NSArray arrayWithObject:validationWarning]];
    
    // set the properties for the entity.
    //
    NSArray *properties = [NSArray arrayWithObjects: dateAttribute, idAttribute, nil];
    [runEntity setProperties:properties];
    
    // Add a Localization Dictionary
    //
    NSMutableDictionary *localizationDictionary = [NSMutableDictionary dictionary];
    [localizationDictionary setObject:@"Date" forKey:@"Property/date/Entity/Run"];
    [localizationDictionary setObject:@"Process ID" forKey:@"Property/processID/Entity/Run"];
    [localizationDictionary setObject:@"Process ID must not be less than 1" forKey:@"ErrorString/Process ID < 1"];
    
    [moModel setLocalizationDictionary:localizationDictionary];
    
    return moModel;
}

NSManagedObjectContext *managedObjectContext()
{
    static NSManagedObjectContext *moContext = nil;
    if (moContext != nil) {
        return moContext;
    }
    
    moContext = [[NSManagedObjectContext alloc] init];
    
    // Create a persistent store coordinator, then set the coordinator for the context.
    //
    NSManagedObjectModel *moModel = managedObjectModel();
    NSPersistentStoreCoordinator *coordinator = [[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel:moModel];
    [moContext setPersistentStoreCoordinator: coordinator];
    
    // Create a new persistent store of the appropriate type. 
    //
    NSString *STORE_TYPE = NSXMLStoreType;
    NSString *STORE_FILENAME = @"CoreDataTutorial.xml";
    
    NSError *error = nil;
    NSURL *url = [applicationLogDirectory() URLByAppendingPathComponent:STORE_FILENAME];
    
    NSPersistentStore *newStore = [coordinator addPersistentStoreWithType:STORE_TYPE
                                                            configuration:nil
                                                                      URL:url
                                                                  options:nil
                                                                    error:&error];
    
    if (newStore == nil) {
        NSLog(@"Store Configuration Failure\n%@", ([error localizedDescription] != nil) ? [error localizedDescription] : @"Unknown Error");
    }

    return moContext;
}

//
//=============================================================================================
//
int main (int argc, const char * argv[])
{
    NSLog(@" === Core Data Tutorial ===");

    // Enable GC
    //
    objc_startCollectorThread();

    NSError *error = nil;
    
    NSManagedObjectModel *moModel = managedObjectModel();
    NSLog(@"The managed object model is defined as follows:\n%@", moModel);
    
    if (applicationLogDirectory() == nil) {
        exit(1);
    }
    
    NSManagedObjectContext *moContext = managedObjectContext();
    
    // Create an Instance of the Run Entity
    //
    NSEntityDescription *runEntity = [[moModel entitiesByName] objectForKey:@"Run"];
    Run *run = [[Run alloc] initWithEntity:runEntity insertIntoManagedObjectContext:moContext];
    NSProcessInfo *processInfo = [NSProcessInfo processInfo];
    run.processID = [processInfo processIdentifier];
    
    if (![moContext save: &error]) {
        NSLog(@"Error while saving\n%@", ([error localizedDescription] != nil) ? [error localizedDescription] : @"Unknown Error");
        exit(1);
    }
    
    // Fetching Run Objects
    //
    NSFetchRequest *request = [[NSFetchRequest alloc] init];
    [request setEntity:runEntity];

    NSSortDescriptor *sortDescriptor = [[NSSortDescriptor alloc] initWithKey:@"date" ascending:YES];
    [request setSortDescriptors:[NSArray arrayWithObject:sortDescriptor]];
    
    error = nil;
    NSArray *array = [moContext executeFetchRequest:request error:&error];
    if ((error != nil) || (array == nil))
    {
        NSLog(@"Error while fetching\n%@", ([error localizedDescription] != nil) ? [error localizedDescription] : @"Unknown Error");
        exit(1);
    }
    
    // Display the Results
    //
    NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
    [formatter setDateStyle:NSDateFormatterMediumStyle];
    [formatter setTimeStyle:NSDateFormatterMediumStyle];
    
    NSLog(@"%@ run history:", [processInfo processName]);
    
    for (run in array)
    {
        NSLog(@"On %@ as process ID %ld", [formatter stringForObjectValue:run.date], run.processID);
    }
    
    return 0;
}















